def fft(data, n, res=512):
  """
  Get spectrum data in log scale using iterpolation.
  :param data: 1D array - sound data array
  :param n: int - total data result
  :param res: int - fft resolution before interpolation
  """
  import numpy as np
  data = np.fft.fft(data, res)
  data = np.abs(data)[:int(res * .8)]
  x = np.array(range(1, len(data) + 1))
  x = np.log(x)
  xlog = np.linspace(np.min(x), np.max(x), n)
  return np.interp(xlog, x, data)


def getSound(t, sound, sr=44100, fps=24):
  """
  Get part of the sound.
  :param t: float - time of the part
  :param sound: 1D array - sound data source
  :param sr: int - sample rate (total sample per second)
  :param fps: int - long of the sound part result is 1/fps
  """
  end = t * sr
  start = end - (sr / fps)
  return sound[int(start): int(end)]


def getAudioData(path):
  """
  Get audio data (sound, sr, fps, duration, andio)
  :param path: string - path of audio file source
  """
  from moviepy.editor import AudioFileClip
  audio = AudioFileClip(path)
  fps = 24
  sound = audio.to_soundarray()
  sr = int(sound.shape[0] / audio.duration)
  return sound, sr, fps, audio.duration, audio


def extract(sound):
  """
  Extract sound to L and R channel.
  :param sound: 2D array - sound with dual channel
  """
  L = sound[:, 0]
  R = sound[:, 1]
  return L, R



def getFile(url, path):
  """
  Download file from url.
  :param url: string - url to download file
  :param path: string - path to save file
  """
  from urllib3 import PoolManager
  http = PoolManager()
  with open(path, 'wb') as f:
    data = http.request('GET', url).data
    f.write(data)
    return True


def ngupukLogo(path="ngupuk.jpg"):
  """
  Download Ngupuk's logo.
  :param path: string - path to save Ngupuk's logo
  """
  url = "https://ngupuk.github.io/logo.jpg"
  return getFile(url, path)


def background(keyword, path="bg.jpg"):
  """
  Download random background from unsplash.
  :param keyword: string - keyword of the image
  :param path: string - path to save background image
  """
  url = "https://source.unsplash.com/featured/?"
  return getFile(url + keyword, path)


def spotifyLogoLight(path="spotify_light.png"):
  """
  Dowload Spotify's light logo
  :param path: string - path to save logo
  """
  url = "https://storage.googleapis.com/pr-newsroom-wp/1/2018/11/Spotify_Logo_RGB_White.png"
  return getFile(url, path)


def spotifyLogoDark(path="spotify_dark.png"):
  """
  Downlad Spotify's dark logo
  :param path: string - path to save logo
  """
  url = "https://storage.googleapis.com/pr-newsroom-wp/1/2018/11/Spotify_Logo_RGB_Black.png"
  return getFile(url, path)


def pathUrl(path, url):
  """Get file path or download before it if not exist.
  :param path: string - path of file
  :param url: string - url to download
  """
  import os.path
  if not os.path.exists(path):
    getFile(url, path)
  return path



class Template1():
  import numpy as _np
  import PIL.Image as _pilImage
  import PIL.ImageDraw as _imgDraw
  import PIL.ImageFont as _imgFont
  import PIL.ImageFilter as _imgFilter
  from igmov import analyzer as _anl
  from igmov import draw as _drw
  from igmov import dl as _dl
  from moviepy.editor import VideoClip as _vc

  _img = _pilImage.new('RGB', (1080, 600))
  _title = ""
  _users = []
  _bg = _pilImage.new('RGB', _img.size)
  _logo = _pilImage.new('RGB', (180, 180))
  _splogo = _pilImage.new('RGB', (100, 30))
  _showSplogo = False

  def useRandomBg(self, keyword):
    bgPath = '__temp__.bg_%s.jpg' % keyword
    try:
      bg = self._pilImage.open(bgPath)
    except:
      self._dl.background(keyword, bgPath)
      bg = self._pilImage.open(bgPath)
    bg = bg.resize(self._bg.size, self._pilImage.ANTIALIAS)
    self._bg = bg
    return self

  def useNgupukLogo(self):
    logoPath = "__temp__.ngupuk.jpg"
    try:
      logo = self._pilImage.open(logoPath)
    except:
      self._dl.ngupukLogo(logoPath)
      logo = self._pilImage.open(logoPath)
    logo = logo.resize(self._logo.size, self._pilImage.ANTIALIAS)
    self._logo = logo
    return self

  def showSpotify(self):
    spPath = '__temp__.spotify_light.png'
    try:
      splogo = self._pilImage.open(spPath)
    except:
      self._dl.spotifyLogoLight(spPath)
      splogo = self._pilImage.open(spPath)
    splogo = splogo.resize(self._splogo.size, self._pilImage.ANTIALIAS)
    self._splogo = splogo
    self._showSplogo = True
    return self

  def _drawSplogo(self):
    if self._showSplogo:
      im = self._img
      font = self._imgFont.truetype('calibri.ttf', 14)
      drw = self._imgDraw.Draw(im)
      drw.text((1080 - 210, 600 - 60), "Listen on", font=font)
      im.paste(self._splogo, (1080 - 150, 600 - 70), self._splogo)
      self._img = im
    return

  def _gen(self):
    bg = self._bg.copy()
    bg = bg.filter(self._imgFilter.GaussianBlur(10))
    bg = self._drw.blackMask(bg)
    mask = self._pilImage.new("RGBA", bg.size)
    drw = self._imgDraw.Draw(mask)
    drw.rectangle((120, 120, 320, 350), fill='black')
    mask = mask.filter(self._imgFilter.GaussianBlur(10))
    drw = self._imgDraw.Draw(mask)
    drw.rectangle((100, 100, 300, 330), fill='white')
    bg.paste(mask, (0, 0), mask)
    bg.paste(self._logo, (110, 110))
    self._img = bg
    self._drawTitle()
    self._drawUsers()
    self._drawSplogo()
    return self._img

  def _drawUsers(self):
    x = 330
    y = 180
    for i, user in enumerate(self._users):
      sx, sy = user.size
      xx = x + i * (sx + 10)
      yy = y - sy
      mask = self._pilImage.new('RGBA', user.size)
      drw = self._imgDraw.Draw(mask)
      drw.ellipse((5, 5, sx - 5, sy - 5), fill='white')
      mask = mask.filter(self._imgFilter.GaussianBlur(2))
      self._img.paste(user, (xx, yy), mask)
    return self

  def _drawTitle(self):
    if self._title:
      font = self._imgFont.truetype('calibri.ttf', 40)
      drw = self._imgDraw.Draw(self._img)
      drw.text((330, 200), self._title, font=font)
      drw.line((330, 250, 450, 250), width=5)
    return self

  def setUsers(self, *path):
    self._users.clear()
    for item in path:
      im = self._pilImage.open(item)
      im = im.resize((70, 70), self._pilImage.ANTIALIAS)
      self._users.append(im)
    return self

  def setBg(self, path):
    bg = self._pilImage.open(path)
    self._bg = bg.resize(self._bg.size, self._pilImage.ANTIALIAS)
    return self

  def setTitle(self, text):
    self._title = text
    return self

  def setLogo(self, path):
    logo = self._pilImage.open(path)
    self._logo = logo.resize(self._logo.size, self._pilImage.ANTIALIAS)
    return self

  def makeVideo(self, audioPath, resultPath):
    self._gen()
    sound, sr, fps, duration, audio = self._anl.getAudioData(audioPath)
    L, _ = self._anl.extract(sound)
    font = self._imgFont.truetype('calibri.ttf', 14)

    def generator(t):
      time = (t // 60, t % 60)
      im = self._img.copy()
      drw = self._imgDraw.Draw(im)
      drw.text((890, 260), "%02i:%02i" % time, font=font)
      data = self._anl.getSound(t, L, sr, fps)
      im = self._drw.lineSpectrum((330, 300),
                                  im,
                                  data,
                                  600,
                                  mode='bottom',
                                  scale=2,
                                  spacing=4)
      im = self._drw.progressBar((328, 280), im, 600, t / duration)
      return self._np.array(im)

    clip = self._vc(generator, duration=duration)
    clip.audio = audio
    clip.write_videofile(resultPath, fps=fps)
    return True



class Igtv1():
  import PIL.Image as _Img
  import PIL.ImageFilter as _Filter
  import PIL.ImageFont as _Font
  import PIL.ImageDraw as _Draw
  import igmov.analyzer as _Anl
  import igmov.dl as _igmDL
  import igmov.draw as _igmDraw
  import numpy as _np
  from moviepy.editor import VideoClip as _vc

  _img = _Img.new('RGB', (600, 1080))
  _bg = _Img.new('RGB', _img.size)
  _logo = _Img.new('RGB', (180, 180))
  _title = ""

  def show(self):
    """
    Get PIL.Image of the first frame.
    """
    return self._gen()

  def setBg(self, path):
    im = self._Img.open(path)
    self._bg = im.resize(self._bg.size, self._Img.ANTIALIAS)
    return self

  def useRandomBg(self, keyword):
    path = '__temp__.bg_%s.jpg' % keyword
    try:
      im = self._Img.open(path)
    except:
      self._igmDL.background(keyword, path)
      im = self._Img.open(path)
    im = im.resize(self._bg.size, self._Img.ANTIALIAS)
    self._bg = im
    return self

  def __drawLogo(self, img):
    x, y = (img.size[0] // 2 - 100, 200)
    mask = self._Img.new('RGBA', img.size)
    drw = self._Draw.Draw(mask)
    drw.rectangle((x + 20, y + 20, x + 220, y + 250), fill='black')
    mask = mask.filter(self._Filter.GaussianBlur(30))
    img.paste(mask, (0, 0), mask)
    drw = self._Draw.Draw(img)
    drw.rectangle((x, y, x + 200, y + 230), fill='white')
    img.paste(self._logo, (x + 10, y + 10))
    return img

  def __drawTitle(self, img):
    x, y = (50, 500)
    font = self._Font.truetype('calibri.ttf', 30)
    drw = self._Draw.Draw(img)
    drw.text((x, y), self._title, font=font)
    drw.line((x, y + 50, x + 100, y + 50), 'white', 5)
    return img

  def _gen(self):
    bg = self._bg.copy()
    bg = bg.filter(self._Filter.GaussianBlur(3))
    bg = self._igmDraw.blackMask(bg)
    bg = self.__drawLogo(bg)
    bg = self.__drawTitle(bg)
    self._img = bg
    return self._img

  def setLogo(self, path):
    im = self._Img.open(path)
    self._logo = im.resize(self._logo.size, self._Img.ANTIALIAS)
    return self

  def useNgupukLogo(self):
    path = '__temp__.ngupuk.jpg'
    try:
      im = self._Img.open(path)
    except:
      self._igmDL.ngupukLogo(path)
      im = self._Img.open(path)
    im = im.resize(self._logo.size, self._Img.ANTIALIAS)
    self._logo = im
    return self

  def setTitle(self, text):
    self._title = text
    return self

  def makeVideo(self, audipath, resultPath):
    self._gen()
    sound, sr, fps, duration, audio = self._Anl.getAudioData(audipath)
    L, _ = self._Anl.extract(sound)

    def generator(t):
      im = self._img.copy()
      data = self._Anl.getSound(t, L, sr, fps)
      im = self._igmDraw.lineSpectrum((50, 800), im, data, 500)
      return self._np.array(im)

    clip = self._vc(generator, duration=duration)
    clip.audio = audio
    clip.write_videofile(resultPath, fps=fps)
    return True



def fullCicrle(pos, image, data):
  """
  Draw full cirlcle spectrum.
  :param pos: (x, y, r) - position and radius of the circle
  :param image: PIL.Image - image to draw
  :param data: 1D array - spectrum data
  """
  from PIL import ImageDraw
  import numpy as np
  im = image.copy()
  x, y, r = pos
  for i, l in enumerate(data):
    x0 = x + r * np.sin(i * 2 * np.pi / len(data))
    x1 = x + (r + l) * np.sin(i * 2 * np.pi / len(data))
    y0 = y - r * np.cos(i * 2 * np.pi / len(data))
    y1 = y - (r + l) * np.cos(i * 2 * np.pi / len(data))
    ImageDraw.Draw(im).line((x0, y0, x1, y1))
  return im


def halfCicrle(pos, image, data):
  """
  Draw half circle spectrum clockwise.
  :param pos: (x, y, r) - position and radius of the circle
  :param image: PIL.Image - image to draw
  :param data: 1D array - spectrum data
  """
  from PIL import ImageDraw
  import numpy as np
  im = image.copy()
  x, y, r = pos
  for i, l in enumerate(data):
    x0 = x + r * np.sin(i * np.pi / len(data))
    x1 = x + (r + l) * np.sin(i * np.pi / len(data))
    y0 = y - r * np.cos(i * np.pi / len(data))
    y1 = y - (r + l) * np.cos(i * np.pi / len(data))
    ImageDraw.Draw(im).line((x0, y0, x1, y1))
  return im


def halfCicrleF(pos, image, data):
  """
  Draw half circle spectrum conter clockwise.
  :param pos: (x, y, r) - position and radius of the circle
  :param image: PIL.Image - image to draw
  :param data: 1D array - spectrum data
  """
  from PIL import ImageDraw
  import numpy as np
  im = image.copy()
  x, y, r = pos
  for i, l in enumerate(data):
    x0 = x - r * np.sin(i * np.pi / len(data))
    x1 = x - (r + l) * np.sin(i * np.pi / len(data))
    y0 = y - r * np.cos(i * np.pi / len(data))
    y1 = y - (r + l) * np.cos(i * np.pi / len(data))
    ImageDraw.Draw(im).line((x0, y0, x1, y1))
  return im


def line(pos, image, data, scale=1, spacing=3, mode='dual'):
  """
  Draw spectrum bars.
  :param pos: (x, y) - position of spectrum on image
  :param image: PIL.image - image to draw
  :param data: 1D array - spectrum data
  :param scale: number - scaling of bar lenght
  :param spacing: int - spacing between bars
  :param mode: dual | bottom | up - direction of bars
  """
  from PIL import ImageDraw
  im = image.copy()
  x, y = pos
  for i, l in enumerate(data):
    tall = l * scale
    xx = x + i * spacing
    if mode == 'bottom':
      y1 = y + tall
      y2 = y
    elif mode == 'up':
      y1 = y
      y2 = y - tall
    else:
      y1 = y + tall
      y2 = y - tall
    ImageDraw.Draw(im).line((xx, y1, xx, y2))
  return im


def lineSpectrum(pos, image, data, width, scale=1, spacing=3, mode="dual"):
  """
  Draw sepectrum bars.
  :param pos: (x, y) - position of spectrum bars on image
  :param image: PIL.Image - image to draw
  :param data: 1D array - sound data
  :param width: int - widht of spectrum on image
  :param scale: number - scaling of bars length
  :param spacing: int - spacing between bars
  :param mode: dual | bottom | up - direction of bars
  """
  from . import analyzer as anl
  count = int(width // spacing)
  spectrum_data = anl.fft(data, count)
  return line(pos, image, spectrum_data, scale, spacing, mode)


def blackMask(image, alpha=.5):
  """
  Draw black mask on image.
  :param image: PIL.Image - image to draw
  :param alpha: float - black mask intensity (0 - 1)
  """
  from PIL import Image
  mask = Image.new('RGB', image.size)
  im = Image.blend(image, mask, alpha)
  return im


def progressBar(pos, image, width, percent):
  """
  Draw proggress bar.
  :param pos: (x, y) - position bars
  :param image: PIL.Image - image to draw
  :param width: int - widht progress bar
  :param percent: float - percentage of proggres (0 - 1)
  """
  from PIL import Image, ImageDraw
  image = image.copy()
  im = Image.new('RGBA', (width + 5, 15))
  gray = (255, 255, 255, 100)
  drw = ImageDraw.Draw(im)
  drw.rectangle((5, 0, width - 5, 10), fill=gray)
  drw.ellipse((0, 0, 10, 10), fill=gray)
  drw.ellipse((width - 10, 0, width, 10), fill=gray)
  drw.ellipse((2, 2, 8, 8), fill='white')
  drw.rectangle((5, 2, 5 + (width - 10) * percent, 8), fill='white')
  endPos = ((width - 10) * percent + 2, 2, (width - 10) * percent + 8, 8)
  drw.ellipse(endPos, fill='white')
  image.paste(im, pos, im)
  return image


def rectangleFillCrop(image):
  """
  Make image in rectangle shape using fill crop.
  :param image: PIL.Image - image source
  """
  image = image.copy()
  x, y = image.size
  n = min(image.size)
  x1 = 0 if x == n else (x - n) // 2
  y1 = 0 if y == n else (y - n) // 2
  image = image.crop((x1, y1, n + x1, n + y1))
  return image
  
def rectangleFill(image, w=None):
  """
  Make Fill Rectangle crop fill
  """
  from PIL import Image
  im = image.copy()
  x, y = im.size
  xx, yy = x//2, y//2
  n = min(x, y) // 2
  im = im.crop((xx - n , yy - n, xx + n, yy + n))
  if(w is not None):
    return im.resize((w, w), Image.ANTIALIAS)
  else:
    return im


class template1(object):
  import numpy as _np
  from moviepy.editor import VideoClip as _vc
  from moviepy.editor import VideoFileClip as _vidFileClip
  import igmov.analyzer as _anl
  import igmov.draw as _igmdraw
  import igmov.dl as _igmdl
  import PIL as _pil
  from PIL import ImageFilter as _imgfilter
  from PIL import ImageDraw as _imgdraw
  from PIL import ImageFont as _imgfont
  import os as _os
  import os.path as _path

  _img = _pil.Image.new('RGB', (800, 800))
  _bg = _pil.Image.new('RGB', _img.size)
  _logo = _pil.Image.new('RGB', (360, 360))
  _splogo = _pil.Image.new('RGBA', (120, 35))
  _title = ""
  _username = ""
  _igLogo = _pil.Image.new('RGBA', (35, 35))
  _showSpotify = False
  _fontName = 'calibri.ttf'
  _vidBG = _vc()
  _isVidBG = False

  if (not _path.exists('__temp__')):
    _os.makedirs('__temp__')

  def useVideoBG(self, path):
    self._vidBG = self._vidFileClip(path)
    self._isVidBG = True
    return self

  def _drawVidBG(self, t):
    if (self._isVidBG):
      now = t % self._vidBG.duration
      img = self._vidBG.get_frame(now)
      img = self._pil.Image.fromarray(img)
      img = self._igmdraw.rectangleFill(img, self._bg.size[0])
      self._bg = img
      self._gen()
    return self._img

  def show(self):
    """
    Get PIL.Image of the first frame.
    """
    return self._gen()

  def _drawBg(self):
    blur = self._imgfilter.GaussianBlur(3)
    img = self._bg.filter(blur)
    self._img.paste(img, (0, 0))
    self._img = self._igmdraw.blackMask(self._img)

  def _drawLogo(self):
    x, y = 200, 100
    logo = self._logo.copy()
    lx, ly = logo.size
    mask = self._pil.Image.new('RGBA', self._img.size)
    drw = self._imgdraw.Draw(mask)
    drw.rectangle((x + 40, y + 40, x + lx + 40, y + ly + 120), fill='black')
    blur = self._imgfilter.GaussianBlur(10)
    mask = mask.filter(blur)
    drw = self._imgdraw.Draw(mask)
    drw.rectangle((x, y, x + lx + 20, y + ly + 90), fill='white')
    mask.paste(logo, (x + 10, y + 10))
    self._img.paste(mask, (0, 0), mask)

  def _drawTitle(self):
    x, y = 215, 480
    font = self._imgfont.truetype(self._fontName, 24)
    drw = self._imgdraw.Draw(self._img)
    drw.text((x, y), self._title, font=font, fill='black')

  def _drawIg(self):
    x, y = self._img.size
    y -= (self._igLogo.size[1] + 20)
    x = 20
    if (self._username):
      font = self._imgfont.truetype(self._fontName, 24)
      self._img.paste(self._igLogo, (x, y), self._igLogo)
      drw = self._imgdraw.Draw(self._img)
      drw.text((x + 40, y + 5), self._username, 'white', font)
    pass

  def _gen(self):
    self._drawBg()
    self._drawLogo()
    self._drawTitle()
    self._drawSpotify()
    self._drawIg()
    return self._img

  def _drawSpotify(self):
    x, y = self._img.size
    x -= self._splogo.size[0] + 120
    y -= self._splogo.size[1] + 20
    bg = self._img
    if self._showSpotify:
      font = self._imgfont.truetype(self._fontName, 24)
      self._imgdraw.Draw(bg).multiline_text((x, y + 8), "Listen On", font=font)
      bg.paste(self._splogo, (x + 100, y), self._splogo)
    self._img = bg

  def setBg(self, path):
    """
    Set background image from local image file.
    :param path: string - path of the image source
    """
    bg = self._pil.Image.open(path)
    self._bg = self._igmdraw.rectangleFill(bg, self._bg.size[0])
    return self

  def setTitle(self, text):
    """
    Set video title.
    :param text: string - video title
    """
    self._title = text
    return self

  def setLogo(self, path):
    """
    Set logo from local storage.
    :param path: string - path of the logo source
    """
    logo = self._pil.Image.open(path)
    self._logo = logo.resize(self._logo.size, self._pil.Image.ANTIALIAS)
    return self

  def showSpotify(self):
    """
    Show Spotify's Logo on frame.
    """
    splogoPath = '__temp__/spotify_light.png'
    try:
      splogo = self._pil.Image.open(splogoPath)
    except:
      self._igmdl.spotifyLogoLight(splogoPath)
      splogo = self._pil.Image.open(splogoPath)
    splogo = splogo.resize(self._splogo.size, self._pil.Image.ANTIALIAS)
    self._splogo = splogo
    self._showSpotify = True
    return self

  def useNgupukLogo(self):
    """
    Use Ngupuk's logo. Automatic download from internet.
    """
    ngupukLogoPath = '__temp__/ngupuk.jpg'
    try:
      ngLogo = self._pil.Image.open(ngupukLogoPath)
    except:
      self._igmdl.ngupukLogo(ngupukLogoPath)
      ngLogo = self._pil.Image.open(ngupukLogoPath)
    ngLogo = ngLogo.resize(self._logo.size, self._pil.Image.ANTIALIAS)
    self._logo = ngLogo
    return self

  def useInstagramLogo(self, username='ngupuk.id'):
    igLogoPath = '__temp__/ig.png'
    try:
      igLogo = self._pil.Image.open(igLogoPath)
    except:
      url = 'https://ngupuk.github.io/static/instagram.png'
      self._igmdl.getFile(url, igLogoPath)
      igLogo = self._pil.Image.open(igLogoPath)
    igLogo = igLogo.resize(self._igLogo.size, self._pil.Image.ANTIALIAS)
    self._username = username
    self._igLogo = igLogo.convert('RGBA')
    return self

  def useRandomBg(self, keyword, always_new=False):
    """
    Use random backgroun from unsplash.
    :param keyword: string - keyword of the image
    """
    bgpath = "__temp__/bg_%s.jpg" % keyword
    if (always_new):
      self._igmdl.background(keyword, bgpath)
      bg = self._pil.Image.open(bgpath)
    else:
      try:
        bg = self._pil.Image.open(bgpath)
      except:
        self._igmdl.background(keyword, bgpath)
        bg = self._pil.Image.open(bgpath)
    bg = bg.resize(self._bg.size, self._pil.Image.ANTIALIAS)
    self._bg = bg
    return self

  def makeVideo(self, audioPath, resultPath):
    """
    Make video file
    :param audioPath: string - path of audio source.
    :param resultPath: string - path of video result.
    """
    self._gen()
    anl = self._anl
    sound, sr, fps, duration, audio = anl.getAudioData(audioPath)
    L, _ = anl.extract(sound)

    def generator(t):
      self._drawVidBG(t)
      im = self._img.copy()
      sp_data = anl.getSound(t, L, sr, fps)
      pos = (200, 560)
      im2 = self._igmdraw.lineSpectrum(pos, im, sp_data, 380, 2, 5, 'bottom')
      return self._np.array(im2)

    clip = self._vc(generator, duration=duration)
    clip.audio = audio
    clip.write_videofile(resultPath, fps=fps)
    return True

  def setFont(self, name):
    self._fontName = name
    return self


class template2(template1):
  def _drawLogo(self):
    x, y = self._img.size
    sx, sy = self._logo.size
    crop = 10
    x = (x - sx) // 2
    y = (y - sy) // 4
    mask = self._pil.Image.new("RGBA", self._logo.size)
    drw = self._imgdraw.Draw(mask)
    drw.ellipse((crop, crop, sx - crop, sy - crop), fill='white')
    blur = self._imgfilter.GaussianBlur(5)
    mask = mask.filter(blur)
    self._img.paste(self._logo, (x, y), mask)

  def _drawTitle(self):
    x, y = (30, 550)
    font = self._imgfont.truetype('calibri.ttf', 40)
    drw = self._imgdraw.Draw(self._img)
    drw.text((x, y), self._title, 'white', font)
    drw.line((x, y + 50, x + 100, y + 50), 'white', 5)

  def makeVideo(self, audioPath, resultPath):
    """
    Make video file
    :param audioPath: string - path of audio source.
    :param resultPath: string - path of video result.
    """
    self._gen()
    sound, sr, fps, duration, audio = self._anl.getAudioData(audioPath)
    L, R = self._anl.extract(sound)

    def generator(t):
      self._drawVidBG(t)
      img = self._img.copy()
      Ldata = self._anl.getSound(t, L, sr, fps)
      Rdata = self._anl.getSound(t, R, sr, fps)
      Lfft = self._anl.fft(Ldata, 100)
      Rfft = self._anl.fft(Rdata, 100)
      im = img
      im = self._igmdraw.halfCicrle((400, 290, 180), im, Lfft * 2)
      im = self._igmdraw.halfCicrleF((400, 290, 180), im, Rfft * 2)
      return self._np.array(im)

    clip = self._vc(generator, duration=duration)
    clip.audio = audio
    clip.write_videofile(resultPath, fps=fps)
    return True


class template3(template1):

  _fontName = 'Gabriola.ttf'
  _logo2 = False

  def _drawLogo(self):
    x, y = (50, 200)
    logo = self._logo.resize((250, 250), self._pil.Image.ANTIALIAS)
    mask = self._pil.Image.new('RGBA', logo.size)
    drw = self._imgdraw.Draw(mask)
    drw.ellipse((5, 5, mask.size[0] - 5, mask.size[1] - 5), fill="white")
    blur = self._imgfilter.GaussianBlur()
    mask = mask.filter(blur)
    self._img.paste(logo, (x, y), mask)
    if (self._logo2):
      pos = (x + 150, y + 150)
      n = (100, 100)
      mask = mask.resize(n)
      logo2 = self._logo2.resize(n, self._pil.Image.ANTIALIAS)
      self._img.paste(mask, pos, mask)
      self._img.paste(logo2, pos, mask)

  def _drawTitle(self):
    x, y = (330, 240)
    titles = self._title.split('\n')
    for i, title in enumerate(titles):
      fontSize = 65 - i * 30
      font = self._imgfont.truetype(self._fontName, fontSize)
      drw = self._imgdraw.Draw(self._img)
      drw.text((x, y + i * 60), title, 'white', font)

  def makeVideo(self, audioPath, resultPath):
    """
    Make video file
    :param audioPath: string - path of audio source.
    :param resultPath: string - path of video result.
    """
    self._gen()
    anl = self._anl
    sound, sr, fps, duration, audio = anl.getAudioData(audioPath)
    L, _ = anl.extract(sound)

    def generator(t):
      im = self._img.copy()
      sp_data = anl.getSound(t, L, sr, fps)
      pos = (40, 580)
      im2 = self._igmdraw.lineSpectrum(pos, im, sp_data, 720, 1.5, 5)
      im2 = self._igmdraw.progressBar((320, 380), im2, 350, t / duration)
      font = self._imgfont.truetype(self._fontName, 30)
      drw = self._imgdraw.Draw(im2)
      drw.text((650, 350), "%02i:%02i" % (t // 60, t % 60), 'white', font)
      return self._np.array(im2)

    clip = self._vc(generator, duration=duration)
    clip.audio = audio
    clip.write_videofile(resultPath, fps=fps)
    return True


class template4(template1):

  _fontName = template3._fontName

  def _drawLogo(self):
    n = 150
    logo = self._logo.resize((n, n), self._pil.Image.ANTIALIAS)
    lx, ly = logo.size
    x, y, r = (self._img.size[0] // 2 - ly // 2, 50, 20)
    mask = self._pil.Image.new('RGBA', logo.size)
    drw = self._imgdraw.Draw(mask)
    drw.rectangle((r, 0, lx - r, ly), 'white')
    drw.rectangle((0, r, lx, ly - r), 'white')
    drw.ellipse((lx - 2 * r, 0, lx, r * 2), 'white')
    drw.ellipse((0, 0, r * 2, r * 2), 'white')
    drw.ellipse((0, ly - r * 2, r * 2, ly), 'white')
    drw.ellipse((lx - r * 2, ly - r * 2, *logo.size), 'white')
    self._img.paste(logo, (x, y), mask)

  def _drawBg(self):
    blur = self._imgfilter.GaussianBlur(10)
    img = self._bg.filter(blur)
    self._img.paste(img, (0, 0))
    self._img = self._igmdraw.blackMask(self._img, .3)

  def _drawTitle(self):
    y = 300
    drw = self._imgdraw.Draw(self._img)
    W, H = self._img.size
    msgs = self._title.split("\n")
    HH = 0
    for i, msg in enumerate(msgs):
      font = self._imgfont.truetype(self._fontName, 50 + i * 100)
      w, h = drw.textsize(msg, font)
      drw.text(((W - w) // 2, (HH + y)), msg, 'white', font, align='center')
      HH += h
    return self

  def makeVideo(self, audioPath, resultPath):
    """
    Make video file
    :param audioPath: string - path of audio source.
    :param resultPath: string - path of video result.
    """
    self._gen()
    anl = self._anl
    sound, sr, fps, duration, audio = anl.getAudioData(audioPath)
    L, R = anl.extract(sound)

    def generator(t):
      self._drawVidBG(t)
      im = self._img.copy()
      y = 125
      sp_data = anl.getSound(t, L, sr, fps)
      sp_data2 = anl.getSound(t, R, sr, fps)
      data = anl.fft(sp_data, 200 // 5)
      data2 = anl.fft(sp_data2, 200 // 5)
      pos = (self._img.size[0] // 2 + 80, y)
      pos2 = (self._img.size[0] // 2 - 80 - 200, y)
      im2 = self._igmdraw.line(pos, im, data, 1.5, 5)
      im2 = self._igmdraw.line(pos2, im2, data2[::-1], 1.5, 5)
      return self._np.array(im2)

    clip = self._vc(generator, duration=duration)
    clip.audio = audio
    clip.write_videofile(resultPath, fps=fps)
    return True
